# ../scripts/custom/gg_ffa_bots/gg_ffa_bots.py

# =============================================================================
# >> IMPORTS
# =============================================================================
# EventScripts Imports
#   ES
from es import gethandlefromindex
from es import getindexprop
from es import getuserid
from es import isbot
from es import ServerVar

# SPE Imports
from spe import detourFunction
from spe import getIndexOfEntity
from spe import HookAction
from spe import HookType
from spe import makeObject
from spe import parseINI
from spe import undetourFunction

# GunGame Imports
#   Addons
from gungame51.core.addons.shortcuts import AddonInfo

# =============================================================================
# >> ADDON REGISTRATION/INFORMATION
# =============================================================================
info = AddonInfo()
info.name = 'gg_ffa_bots'
info.title = 'GG Free For All BOTS'
info.author = "L'In20Cible"
info.version = '1.1'
info.requires = ['gg_ffa']

# =============================================================================
# >> PUBLIC VARIABLE
# =============================================================================
ServerVar(info.name + '_version', info.version,
        info.title + ' Version').set(info.version)
ServerVar(info.name + '_version').makepublic()

# =============================================================================
# >> GLOBAL VARIABLES
# =============================================================================
hacked_teams = dict()


# =============================================================================
# >> LOAD & UNLOAD
# =============================================================================
def load():
    '''Called when the addon is loaded'''

    # Parse the .ini file to get all the signatures
    parseINI('gungame51/scripts/custom/gg_ffa_bots/signatures.ini')

    # Detour all of the functions
    detourFunction('GetTeamNumber', HookType.Pre, PreGetTeamNumber)
    detourFunction('InSameTeam', HookType.Pre, PreInSameTeam)
    detourFunction('OnAudibleEvent', HookType.Pre, PreOnAudibleEvent)
    detourFunction('OnPlayerRadio', HookType.Pre, PreOnPlayerRadio)


def unload():
    '''Called when the addon is unloaded'''

    # Undetour all of the functions
    undetourFunction('GetTeamNumber', HookType.Pre, PreGetTeamNumber)
    undetourFunction('InSameTeam', HookType.Pre, PreInSameTeam)
    undetourFunction('OnAudibleEvent', HookType.Pre, PreOnAudibleEvent)
    undetourFunction('OnPlayerRadio', HookType.Pre, PreOnPlayerRadio)


# =============================================================================
# >> DETOUR CALLBACKS
# =============================================================================
def PreGetTeamNumber(args):
    '''Called each time a bot's team number is retrieved'''

    # Is the player in the hacked_teams dictionary?
    if not args[0] in hacked_teams:

        # If not, allow the function to continue normally
        return (HookAction.Continue, 0)

    # Get the team to return
    value = hacked_teams[args[0]]

    # Delete the player from the dictionary
    del hacked_teams[args[0]]

    # Override the function, returning the new value
    return (HookAction.Override, value)


def PreInSameTeam(args):
    '''Called when a check is made to see if 2 players are on the same team'''

    # Is this 2 different players?
    if args[0] != args[1]:

        # Is the first player a bot?
        if isbot(getuserid(gethandlefromindex(getIndexOfEntity(args[0])))):

            # Always return False for Bots
            return (HookAction.Override, False)

    # Allow the function to continue as normal
    return (HookAction.Continue, 0)


def PreOnAudibleEvent(args):
    '''Called when an audible event occurs'''

    # Is there a player pointer?
    if args[1]:

        # Get the first player's team
        teamid = getindexprop(
            getIndexOfEntity(args[1]), 'CBaseEntity.m_iTeamNum')

        # Are the 2 players on the same team?
        if teamid == getindexprop(
          getIndexOfEntity(args[7]), 'CBaseEntity.m_iTeamNum'):

            # If so, store the first player with the opposing team
            hacked_teams[args[1]] = 5 - teamid

    # Allow the function to continue as normal
    return (HookAction.Continue, 0)


def PreOnPlayerRadio(args):
    '''Called when a radio message is going to be sent'''

    # Always override the function
    return (HookAction.Override, 0)
